@@ -1,5 +1,5 @@
 target.arrival.sf <-
-function(xdata,ydata,date,t,rts,g,w=NULL,sg="ssm",ftype="d"){
+function(xdata,ydata,date,t,rts="crs",g,wd=NULL,sg="ssm",ftype="d",cv="convex"){
   
   # Initial checks
   if(is.na(match(rts,c("crs","vrs","irs","drs")))){stop('rts must be "crs", "vrs", "irs", or "drs".')}
@@ -7,12 +7,14 @@ function(xdata,ydata,date,t,rts,g,w=NULL,sg="ssm",ftype="d"){
   if(is.na(match(ftype,c("d","s")))){stop('ftype must be either "d" or "s".')}
   if(t<=min(date)){stop('t is earlier than dataset.')}
   if(max(date)<=t){stop('t is later than dataset.')}
+  if(is.na(match(cv,c("convex","fdh")))){stop('cv must be "convex" or "fdh".')}
   
   # Parameters
   xdata<-as.matrix(xdata);ydata<-as.matrix(ydata);date<-as.matrix(date);g<-as.matrix(g) # format input data as matrix
   n<-nrow(xdata); m<-ncol(xdata); s<-ncol(ydata)
-  if(is.null(w)) w<-matrix(c(0),ncol=s) else w<-as.matrix(w)
+  if(is.null(wd)) wd<-matrix(c(0),ncol=s) else wd<-as.matrix(wd)
   o<-matrix(c(1:n),ncol=1) # original data order
+  if(cv=="fdh") rts<-"vrs"
   
   # Sort data ascending order
   x<-matrix(c(xdata[order(date),]),ncol=m)
@@ -45,14 +47,14 @@ function(xdata,ydata,date,t,rts,g,w=NULL,sg="ssm",ftype="d"){
   g_t<-matrix(g[1:e,],nrow=e)
   
   # SF internal function 
-  dm.sf.internal<-function(xdata,ydata,rts,g,w,se=0,sg,date,a,z){
+  dm.sf.internal<-function(xdata,ydata,rts,g,wd,se=0,sg,date,a,z,cv){
     
     # Load library
     # library(lpSolveAPI)
     
     # Parameters
     n<-nrow(xdata); m<-ncol(xdata); s<-ncol(ydata)
-    if(is.null(w)){w<-matrix(c(0),ncol=s)}
+    if(is.null(wd)){wd<-matrix(c(0),ncol=s)}
     
     # Data frames
     results.efficiency<-matrix(rep(NA,n),nrow=n,ncol=1)
@@ -74,15 +76,18 @@ function(xdata,ydata,date,t,rts,g,w=NULL,sg="ssm",ftype="d"){
       if(rts=="irs"){add.constraint(lp.sf,c(rep(1,n*2)),indices=c(1:(n*2)),">=",1)}
       if(rts=="drs"){add.constraint(lp.sf,c(rep(1,n*2)),indices=c(1:(n*2)),"<=",1)}
       
+      # Set type
+      if(cv=="fdh"){set.type(lp.sf,1:n,"binary")}
+      
       # Mu
-      if(rts=="crs"||rts=="drs"||sum(w)==0){add.constraint(lp.sf,c(rep(1,n)),indices=c((n+1):(n+n)),"=",0)}
+      if(rts=="crs"||rts=="drs"||sum(wd)==0){add.constraint(lp.sf,c(rep(1,n)),indices=c((n+1):(n+n)),"=",0)}
       
       # Input constraints
       for(i in 1:m){add.constraint(lp.sf,c(xdata[,i],xdata[,i],g[k,i],1),indices=c(1:n,(n+1):(n+n),n+n+1,n+n+1+i),"=",xdata[k,i])}
       
       # Output constraints
       for(r in 1:s){
-        if(w[1,r]==1){
+        if(wd[1,r]==1){
           add.constraint(lp.sf,c(ydata[,r],ydata[,r],g[k,m+r]),indices=c(1:n,(n+1):(n+n),n+n+1),"=",ydata[k,r])
           add.constraint(lp.sf,c(1),indices=c(n+n+1+m+r),"=",0)
         }else{add.constraint(lp.sf,c(ydata[,r],-g[k,m+r],-1),indices=c(1:n,n+n+1,n+n+1+m+r),"=",ydata[k,r])}
@@ -141,7 +146,7 @@ function(xdata,ydata,date,t,rts,g,w=NULL,sg="ssm",ftype="d"){
     y_f<-rbind(y_t,y[i,])
     g_f<-rbind(g_t,g[i,])
     # Run SF
-    temp<-dm.sf.internal(x_f,y_f,rts,g_f,w,se=1,sg,d,e+1,e+1)
+    temp<-dm.sf.internal(x_f,y_f,rts,g_f,wd,se=1,sg,d,e+1,e+1,cv)
     # Save eff_t
     eff_t[i,]<-temp$eff[e+1,]
     lambda[i,1:e]<-temp$lambda[e+1,1:e]
@@ -152,7 +157,7 @@ function(xdata,ydata,date,t,rts,g,w=NULL,sg="ssm",ftype="d"){
   if(ftype=="s"){ed[,1]<-t}
   
   # Calc iRoC
-  roc<-roc.sf(xdata,ydata,date,t,rts,g,w,sg,ftype)
+  roc<-roc.sf(xdata,ydata,date,t,rts,g,wd,sg,ftype,cv=cv)
   roc_local<-roc$roc_local;roc_local_bi<-ifelse(is.na(roc_local),0,1);roc_avg<-roc$roc_avg
   for(i in (e+1):n){roc_ind[i,1]<-sum(roc_local[1:e,]*lambda[i,1:e],na.rm=TRUE)/sum(lambda[i,1:e]*roc_local_bi[1:e,])}
 
